/***************************************************************************
 *   Copyright (C) 2004 by Stefan Baur                                     *
 *   stefan@c0n.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qpixmap.h>

#include "kglwater.h"

KGLWater::KGLWater(QWidget *parent, const char *name)
		: QGLWidget(parent, name)
	{
	setWFlags(getWFlags()|WNoAutoErase);
	}


KGLWater::~KGLWater()
	{
	water.quit();
	}



void KGLWater::readSettings(KConfig* config)
	{
	QString imgFile=config->readPathEntry( "background", "desktop" ); // "/home/stefan/Documents/Bilder/Wallpaper/blue-sun-m.jpg");

	water.Settings.realtime=config->readBoolEntry("realtime", true);
	water.Settings.water_res_x=config->readNumEntry("water_res_x",128);
	water.Settings.water_res_y=config->readNumEntry("water_res_y",128);
	water.Settings.water_step_time=1.0/(double)config->readNumEntry("water_cps",24);

	water.Settings.timer_ms=config->readNumEntry("timer_ms",32);
	startTimer( water.Settings.timer_ms );


	if(imgFile=="desktop")
		{
		bgImage=QPixmap::grabWindow(QApplication::desktop()->winId());
		}
	else
		{
		if(bgImage.load( imgFile ))
			{

			}
		}
//	setBackgroundColor( black );
//	erase();
	}


void KGLWater::initializeGL()
	{
	int texSize;

	water.init();

	glGetIntegerv(GL_MAX_TEXTURE_SIZE,&texSize);
	if(texSize<64)texSize=64;
	if(texSize>1024)texSize=1024;

	bgImage=bgImage.smoothScale(texSize,texSize,QImage::ScaleFree);
	bgImage=QGLWidget::convertToGLFormat(bgImage);

	glGenTextures(1,&water.Settings.bgTexID);
	glBindTexture(GL_TEXTURE_2D,water.Settings.bgTexID);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
	glEnable(GL_TEXTURE_2D);
	glTexImage2D(GL_TEXTURE_2D,0,3,bgImage.width(),bgImage.height(),0,GL_RGBA,GL_UNSIGNED_BYTE,bgImage.bits() );

	}


void KGLWater::resizeGL( int width, int height )
	{
	glViewport( 0, 0, width, height );

	glMatrixMode(GL_PROJECTION);
	 glLoadIdentity();
	 glOrtho( -1, 1, -1, 1, 5.0, 60.0 );

	glMatrixMode(GL_MODELVIEW);
	 glLoadIdentity();
	}


void KGLWater::paintGL()
	{
	water.render();
	}

void KGLWater::timerEvent(QTimerEvent*)
	{
	if(water.update())updateGL();
	}


#include "kglwater.moc"
