/***************************************************************************
 *   Copyright (C) 2004 by Stefan Baur                                     *
 *   stefan@c0n.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>
#include <kapplication.h>
#include <klocale.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qcolor.h>
#include <kurlrequester.h>
#include <kpushbutton.h>
#include <kglobal.h>
#include "kglwatersaver.h"
#include "kglwatersaverui.h"

//! libkscreensaver interface
extern "C"
{
    const char *kss_applicationName = "kglwatersaver.kss";
    const char *kss_description = I18N_NOOP( "KGLWaterSaver" );
    const char *kss_version = "2.2.0";

    KGLWaterSaver *kss_create( WId id )
    {
        KGlobal::locale()->insertCatalogue("kglwatersaver");
        return new KGLWaterSaver( id );
    }

    QDialog *kss_setup()
    {
        KGlobal::locale()->insertCatalogue("kglwatersaver");
        return new KGLWaterSaverSetup();
    }
}

//-----------------------------------------------------------------------------
//! dialog to setup screen saver parameters
KGLWaterSaverSetup::KGLWaterSaverSetup( QWidget *parent, const char *name )
        : KGLWaterSaverUI( parent, name, TRUE )
{
    //Connect your signals and slots here to configure the screen saver.
    connect( OkayPushButton, SIGNAL( released() ),
             SLOT( slotOkPressed() ) );
    connect( CancelPushButton, SIGNAL( released() ),
             SLOT( slotCancelPressed() ) );

	readSettings();
}


//! read settings from config file
void KGLWaterSaverSetup::readSettings()
{
	QString img;
    KConfig *config = KGlobal::config();
    config->setGroup( "Settings" );

	cbRealtime->setChecked(config->readBoolEntry( "realtime", true ));
	sbWaterX->setValue(config->readNumEntry( "water_res_x", 128 ));
	sbWaterY->setValue(config->readNumEntry( "water_res_y", 128 ));
	sbWaterCPS->setValue(config->readNumEntry( "water_cps", 24 ));
	sbTimerMS->setValue(config->readNum64Entry( "timer_ms", 32 ));
	img=config->readPathEntry( "background", "desktop" );
	if(img=="desktop")
		{
		rbImageFile->setChecked(false);
		rbImageDesktop->setChecked(true);
		}
	else
		{
		rbImageFile->setChecked(true);
		rbImageDesktop->setChecked(false);
		urlImage->setURL(img);
		}
}


//! Ok pressed - save settings and exit
void KGLWaterSaverSetup::slotOkPressed()
{
	QString img="desktop";

    KConfig *config = KGlobal::config();
    config->setGroup( "Settings" );
	config->writeEntry( "realtime", cbRealtime->isChecked() );
	config->writeEntry( "water_res_x", sbWaterX->value() );
	config->writeEntry( "water_res_y", sbWaterY->value() );
	config->writeEntry( "water_cps", sbWaterCPS->value() );
	config->writeEntry( "timer_ms", sbTimerMS->value() );
	if(rbImageDesktop->isChecked())config->writePathEntry( "background", img);
	else config->writePathEntry( "background", urlImage->url() );
    config->sync();

    accept();
}

void KGLWaterSaverSetup::slotCancelPressed()
{
    reject();
}
//-----------------------------------------------------------------------------









KGLWaterSaver::KGLWaterSaver(WId id):KScreenSaver( id )
	{
	setWFlags( getWFlags()|WNoAutoErase );

	water = new KGLWater;
	readSettings();
	embed(water);

	//setBackgroundColor( black );
	//erase();
	water->show();
	}

KGLWaterSaver::~KGLWaterSaver()
	{
	delete water;
	}

//! read configuration settings from config file
void KGLWaterSaver::readSettings()
	{
	KConfig *config = KGlobal::config();
	config->setGroup( "Settings" );
	water->readSettings(config);
	}

